@@ -3,6 +3,7 @@
 import logging
 from pathlib import Path
 import zipfile
+from typing import Set, Dict, Optional, Union, List, Any, Iterable
 
 import pandas as pd
 import numpy as np
@@ -20,7 +21,7 @@ from .pyscallist import PyscalList
 logger = logging.getLogger(__name__)
 
 
-def slicedict(dct, keys):
+def slicedict(dct: dict, keys: Iterable):
     """Slice a dictionary for a set of keys.
     Keys not existing will be ignored.
     """
@@ -33,7 +34,7 @@ def slicedict(dct, keys):
 # for LET parameters, where the API is simplified to 'l', 'e' and 't'.
 # We are case *insensitive* in this factory class, so everything here should
 # be lower case
-WO_INIT = ["swirr", "swl", "swcr", "sorw", "h", "tag"]
+WO_INIT = ["swirr", "swl", "swcr", "sorw", "sgrw", "h", "tag"]
 WO_COREY_WATER = ["nw"]
 WO_WATER_ENDPOINTS = ["krwmax", "krwend"]
 WO_COREY_OIL = ["now"]
@@ -55,11 +56,15 @@ WO_SIMPLE_J_PETRO = [
     "drho",
 ]  # "g" is optional
 
-GO_INIT = ["swirr", "sgcr", "sorg", "swl", "krgendanchor", "h", "tag"]
+GO_INIT = ["swirr", "sgcr", "sorg", "sgro", "swl", "krgendanchor", "h", "tag"]
 GO_COREY_GAS = ["ng"]
 GO_GAS_ENDPOINTS = ["krgend", "krgmax"]
 GO_COREY_OIL = ["nog"]
-GO_OIL_ENDPOINTS = ["kroend", "krogend"]  # krogend is deprecated in favour of kroend
+GO_OIL_ENDPOINTS = [
+    "kroend",
+    "kromax",
+    "krogend",
+]
 GO_LET_GAS = ["lg", "eg", "tg"]
 GO_LET_OIL = ["log", "eog", "tog"]
 
@@ -86,7 +91,6 @@ WOG_INIT = ["swirr", "swl", "swcr", "sorw", "sorg", "sgcr", "h", "tag"]
 DEPRECATED = {
     "krogend": "and will be used as kroend for GasOil",
     "krowend": "and will be used as kroend for WaterOil",
-    "kromax": "and will be ignored",
 }
 
 
@@ -114,7 +118,9 @@ class PyscalFactory(object):
     """
 
     @staticmethod
-    def create_water_oil(params=None, fast=False):
+    def create_water_oil(
+        params: Optional[Dict[str, float]] = None, fast: bool = False
+    ) -> WaterOil:
         """Create a WaterOil object from a dictionary of parameters.
 
         Parameterization (Corey/LET) is inferred from presence
@@ -132,14 +138,14 @@ class PyscalFactory(object):
         translated to 'l')
 
         Recognized parameters:
-          swirr, swl, swcr, sorw, h, tag, nw, now, krwmax, krwend,
+          swirr, swl, swcr, sorw, sgrw, h, tag, nw, now, krwmax, krwend,
           lw, ew, tw, low, eow, tow, lo, eo, to, kroend,
           a, a_petro, b, b_petro, poro_ref, perm_ref, drho,
           a, b, poro, perm, sigma_costau
 
         Args:
-            params (dict): Dictionary with parameters describing the WaterOil object.
-            fast (bool): If fast-mode should be set for constructed object.
+            params: Dictionary with parameters describing the WaterOil object.
+            fast: If fast-mode should be set for constructed object.
         """
         if not params:
             params = dict()
@@ -156,7 +162,7 @@ class PyscalFactory(object):
         # Allowing sending in NaN values, delete those keys.
         params = filter_nan_from_dict(params)
 
-        usedparams = set()
+        usedparams: Set[str] = set()
 
         # Check if we should initialize swl from a swlheight parameter:
         if set(WO_SWL_FROM_HEIGHT).issubset(params):
@@ -202,7 +208,9 @@ class PyscalFactory(object):
             params["swcr"] = params["swl"] + params[WO_SWCR_ADD[0]]
 
         # No requirements to the base objects, defaults are ok.
-        wateroil = WaterOil(**slicedict(params, WO_INIT), fast=fast)
+        wateroil = WaterOil(
+            **PyscalFactory.alias_sgrw(slicedict(params, WO_INIT)), fast=fast
+        )
         usedparams = usedparams.union(set(slicedict(params, WO_INIT).keys()))
         logger.debug(
             "Initialized WaterOil object from parameters %s", str(list(usedparams))
@@ -226,10 +234,6 @@ class PyscalFactory(object):
                 "Added LET water to WaterOil object from parameters %s",
                 str(params_let_water.keys()),
             )
-        else:
-            logger.warning(
-                "Missing or ambiguous parameters for water curve in WaterOil object"
-            )
 
         # Oil curve:
         params_corey_oil = slicedict(params, WO_COREY_OIL + WO_OIL_ENDPOINTS)
@@ -269,10 +273,6 @@ class PyscalFactory(object):
                 "Added LET water to WaterOil object from parameters %s",
                 str(params_let_oil.keys()),
             )
-        else:
-            logger.warning(
-                "Missing or ambiguous parameters for oil curve in WaterOil object"
-            )
 
         # Capillary pressure:
         params_simple_j = slicedict(params, WO_SIMPLE_J + ["g"])
@@ -300,7 +300,9 @@ class PyscalFactory(object):
         return wateroil
 
     @staticmethod
-    def create_gas_oil(params=None, fast=False):
+    def create_gas_oil(
+        params: Optional[Dict[str, float]] = None, fast: bool = False
+    ) -> GasOil:
         """Create a GasOil object from a dictionary of parameters.
 
         Parameterization (Corey/LET) is inferred from presence
@@ -316,15 +318,12 @@ class PyscalFactory(object):
 
         Recognized parameters:
           swirr, sgcr, sorg, swl, krgendanchor, h, tag,
-          ng, krgend, krgmax, nog, kroend,
+          ng, krgend, krgmax, nog, kroend, kromax
           lg, eg, tg, log, eog, tog
 
         Args:
-            params (dict): Dictionary with parameters describing the GasOil object.
-            fast (bool): If fast-mode should be set for constructed object.
-
-        Returns:
-            GasOil
+            params: Dictionary with parameters describing the GasOil object.
+            fast: If fast-mode should be set for constructed object.
         """
         if not params:
             params = dict()
@@ -341,7 +340,7 @@ class PyscalFactory(object):
         # Allowing sending in NaN values, delete those keys.
         params = filter_nan_from_dict(params)
 
-        usedparams = set()
+        usedparams: Set[str] = set()
         # No requirements to the base objects, defaults are ok.
         gasoil = GasOil(**slicedict(params, GO_INIT), fast=fast)
         usedparams = usedparams.union(set(slicedict(params, GO_INIT).keys()))
@@ -408,7 +407,9 @@ class PyscalFactory(object):
         return gasoil
 
     @staticmethod
-    def create_water_oil_gas(params=None, fast=False):
+    def create_water_oil_gas(
+        params: Optional[Dict[str, float]] = None, fast: bool = False
+    ) -> WaterOilGas:
         """Create a WaterOilGas object from a dictionary of parameters
 
         Parameterization (Corey/LET) is inferred from presence
@@ -418,8 +419,8 @@ class PyscalFactory(object):
         of supported parameters (case insensitive)
 
         Params:
-            params (dict): Dictionary with parameters describing the WaterOilGas object.
-            fast (bool): If fast-mode should be set for constructed object.
+            params: Dictionary with parameters describing the WaterOilGas object.
+            fast: If fast-mode should be set for constructed object.
         """
         if not params:
             params = dict()
@@ -431,6 +432,7 @@ class PyscalFactory(object):
         # For case insensitiveness, all keys are converted to lower case:
         params = {key.lower(): value for (key, value) in params.items()}
 
+        wateroil: Optional[WaterOil]
         if sufficient_water_oil_params(params, failhard=False):
             wateroil = PyscalFactory.create_water_oil(params, fast=fast)
         else:
@@ -439,9 +441,10 @@ class PyscalFactory(object):
 
         # If the swl in WaterOil was initialized with swlheight,
         # ensure that result is passed on to the GasOil object:
-        if "swl" not in params and "swlheight" in params:
+        if "swl" not in params and "swlheight" in params and wateroil is not None:
             params["swl"] = wateroil.swl
 
+        gasoil: Optional[GasOil]
         if sufficient_gas_oil_params(params, failhard=False):
             gasoil = PyscalFactory.create_gas_oil(params, fast=fast)
         else:
@@ -457,23 +460,22 @@ class PyscalFactory(object):
         wateroilgas.gasoil = gasoil  # This might be None
         if not wateroilgas.selfcheck():
             raise ValueError(
-                ("Incomplete WaterOilGas object, some parameters missing to factory")
+                f"Inconsistent WaterOilGas object. Bug? Input was {params}"
             )
         return wateroilgas
 
     @staticmethod
-    def create_gas_water(params=None, fast=False):
+    def create_gas_water(
+        params: Optional[Dict[str, float]] = None, fast: bool = False
+    ) -> GasWater:
         """Create a GasWater object.
 
         Parameterization (Corey/LET) is inferred from presence
         of certain parameters in the dictionary.
 
         Args:
-            params (dict): Dictionary with parameters for GasWater.
-            fast (bool): If fast-mode should be set for constructed object.
-
-        Returns:
-            GasWater
+            params: Dictionary with parameters for GasWater.
+            fast: If fast-mode should be set for constructed object.
         """
         if not params:
             params = dict()
@@ -498,12 +500,19 @@ class PyscalFactory(object):
         wog_params["nog"] = 1
         wog_params["now"] = 1
         wog = PyscalFactory.create_water_oil_gas(wog_params, fast=fast)
+        assert wog.wateroil is not None
+        assert wog.gasoil is not None
         gaswater.wateroil = wog.wateroil
         gaswater.gasoil = wog.gasoil
         return gaswater
 
     @staticmethod
-    def create_scal_recommendation(params, tag="", h=None, fast=False):
+    def create_scal_recommendation(
+        params: Dict[str, Dict[str, float]],
+        tag: str = "",
+        h: Optional[float] = None,
+        fast: bool = False,
+    ) -> SCALrecommendation:
         """
         Set up a SCAL recommendation curve set from input as a
         dictionary of dictionary.
@@ -523,17 +532,14 @@ class PyscalFactory(object):
         and high.
 
         Args:
-            params (dict): keys low, base and high.
+            params: keys low, base and high.
                 The value for "low" must be a new dictionary with saturation
                 endpoints and LET/Corey parameters, as you would feed it to
                 the create_water_oil_gas() factory function, and then similarly
                 for base and high.
-            tag (string): String to be used as the tag, will end up in comments.
-            h (float): Saturation step length
-            fast (bool): If fast-mode should be set for constructed object.
-
-        Returns:
-            SCALrecommendation
+            tag: String to be used as the tag, will end up in comments.
+            h: Saturation step length
+            fast: If fast-mode should be set for constructed object.
         """
         if not isinstance(params, dict):
             raise ValueError("Input must be a dictionary (of dictionaries)")
@@ -571,6 +577,10 @@ class PyscalFactory(object):
             [sufficient_water_oil_params(params[case]) for case in params.keys()]
         )
 
+        wog_low: Union[WaterOilGas, GasWater]
+        wog_base: Union[WaterOilGas, GasWater]
+        wog_high: Union[WaterOilGas, GasWater]
+
         if wateroil or gasoil:
             try:
                 wog_low = PyscalFactory.create_water_oil_gas(params["low"], fast=fast)
@@ -608,7 +618,9 @@ class PyscalFactory(object):
         return scal
 
     @staticmethod
-    def load_relperm_df(inputfile, sheet_name=None):
+    def load_relperm_df(
+        inputfile: Union[str, pd.DataFrame], sheet_name: Optional[str] = None
+    ) -> pd.DataFrame:
         """Read CSV or XLSX from file and return scal/relperm data
         a dataframe.
 
@@ -616,19 +628,19 @@ class PyscalFactory(object):
         Ensures case-insensitiveness SATNUM, CASE, TAG and COMMENT
 
         Merges COMMENT into TAG column, as only TAG is picked up downstream.
-        Adds a prexix "SATNUM <number>" to all tags.
+        Adds a prefix "SATNUM <number>" to all tags.
 
         All strings in CASE column are converted to lowercase. Applies
         aliasing in the CASE column so that "pessimistic" and "pess" map to
         "low", and "optimistic" and "opt" map to "high".
 
         Args:
-            inputfile (str or pd.DataFrame): Filename for XLSX or CSV file, or a
+            inputfile: Filename for XLSX or CSV file, or a
                 pandas DataFrame.
-            sheet_name (str): Sheet-name, only used when loading xlsx files.
+            sheet_name: Sheet-name, only used when loading xlsx files.
 
         Returns:
-            pd.DataFrame. To be handed over to pyscal list factory methods.
+            To be handed over to pyscal list factory methods.
             Empty dataframe in case of errors (messages will be logged).
         """
         if isinstance(inputfile, (str, Path)) and Path(inputfile).is_file():
@@ -642,7 +654,7 @@ class PyscalFactory(object):
                     "Sheet name only relevant for XLSX files, ignoring %s", sheet_name
                 )
             excel_engines = {"xls": "xlrd", "xlsx": "openpyxl"}
-            if sheet_name:
+            if tabular_file_format != "csv" and sheet_name:
                 try:
                     input_df = pd.read_excel(
                         inputfile,
@@ -655,10 +667,10 @@ class PyscalFactory(object):
                         inputfile,
                         sheet_name,
                     )
-                except KeyError as error:
-                    logger.error("Non-existing sheet-name %s provided?", sheet_name)
-                    logger.error(str(error))
-                    return pd.DataFrame()
+                except (KeyError, ValueError) as err:
+                    raise ValueError(
+                        f"Non-existing sheet-name {sheet_name} provided."
+                    ) from err
             elif tabular_file_format.startswith("xls"):
                 input_df = pd.read_excel(
                     inputfile, engine=excel_engines[tabular_file_format]
@@ -738,10 +750,9 @@ class PyscalFactory(object):
             raise ValueError("SATNUM must start at 1")
 
         if max(input_df["SATNUM"]) != len(input_df["SATNUM"].unique()):
-            logger.error(
+            raise ValueError(
                 "Missing SATNUMs? Max SATNUM is not equal to number of unique SATNUMS"
             )
-            raise ValueError
         if "CASE" not in input_df and len(input_df["SATNUM"].unique()) != len(input_df):
             raise ValueError("Non-unique SATNUMs?")
         # If we are in a SCAL recommendation setting
@@ -774,13 +785,10 @@ class PyscalFactory(object):
 
         # Check that we are able to make something out of the first row:
         firstrow = input_df.iloc[0, :]
-        error = False
-        try:
-            wo_ok = sufficient_water_oil_params(firstrow)
-            go_ok = sufficient_gas_oil_params(firstrow)
-            gw_ok = sufficient_gas_water_params(firstrow)
-        except ValueError:
-            error = True
+        error: bool = False
+        wo_ok = sufficient_water_oil_params(firstrow)
+        go_ok = sufficient_gas_oil_params(firstrow)
+        gw_ok = sufficient_gas_water_params(firstrow)
         if error or not wo_ok and not go_ok and not gw_ok:
             raise ValueError(
                 "Can't make neither WaterOil, GasOil or GasWater from "
@@ -795,7 +803,39 @@ class PyscalFactory(object):
         return input_df.sort_values("SATNUM")
 
     @staticmethod
-    def remap_validate_cases(casevalues):
+    def alias_sgrw(params: Dict[str, Any]) -> Dict[str, Any]:
+        """Allow sgrw as an alias for sorw by remapping a
+        sgrw value to a sorw value in an incoming dict.
+
+        Will error hard of sorw already exists and is not nan.
+
+        This aliasing is relevant when GasWater is modelled
+        as three-phase to allow for condensate forming, and mirrors
+        GasWater.__init__() which performs the same aliasing.
+
+        Args:
+            params: Keys must be lower case.
+        """
+        if "sgrw" in params:
+            if "sorw" not in params or pd.isnull(params["sorw"]):
+                params_copy = dict(params)
+                params_copy["sorw"] = params["sgrw"]
+                del params_copy["sgrw"]
+                return params_copy
+            else:
+                if np.isclose(params["sgrw"], params["sorw"]):
+                    params_copy = dict(params)
+                    del params_copy["sgrw"]
+                    return params_copy
+                else:
+                    raise ValueError(
+                        f"sgrw ({params['sgrw']}) must equal sorw ({params['sorw']}) "
+                        "when both are supplied to WaterOil."
+                    )
+        return params
+
+    @staticmethod
+    def remap_validate_cases(casevalues: List[str]) -> List[str]:
         """Remap values in the CASE column so that we can use aliases.
 
         All values are first made lower case, then
@@ -806,7 +846,7 @@ class PyscalFactory(object):
         if we don't have exactly three unique values.
 
         Args:
-            casevalues (list of str): values to remap.
+            casevalues: values to remap.
         """
         accepted = ["low", "base", "high"]
         aliases = {
@@ -829,14 +869,16 @@ class PyscalFactory(object):
         return remapped
 
     @staticmethod
-    def create_scal_recommendation_list(input_df, h=None, fast=False):
+    def create_scal_recommendation_list(
+        input_df: pd.DataFrame, h: Optional[float] = None, fast: bool = False
+    ) -> PyscalList:
         """Requires SATNUM and CASE to be defined in the input data
 
         Args:
-            input_df (pd.DataFrame): Input data, should have been processed
+            input_df: Input data, should have been processed
                 through load_relperm_df().
-            h (float): Saturation step-value
-            fast (bool): If fast-mode should be set for constructed object
+            h: Saturation step-value
+            fast: If fast-mode should be set for constructed object
 
         Returns:
             PyscalList, consisting of SCALrecommendation objects
@@ -865,15 +907,17 @@ class PyscalFactory(object):
         return scal_l
 
     @staticmethod
-    def create_pyscal_list(relperm_params_df, h=None, fast=False):
+    def create_pyscal_list(
+        relperm_params_df: pd.DataFrame, h: Optional[float] = None, fast: bool = False
+    ):
         """Create WaterOilGas, WaterOil, GasOil or GasWater list
         based on what is available
 
         Args:
-            relperm_params_df (pd.DataFrame): Input data, should have been processed
+            relperm_params_df: Input data, should have been processed
                 through load_relperm_df().
-            h (float): Saturation step-value
-            fast (bool): If fast-mode should be set for constructed object
+            h: Saturation step-value
+            fast: If fast-mode should be set for constructed object
 
         Returns:
             PyscalList, consisting of either WaterOil, GasOil or WaterOilGas objects
@@ -894,15 +938,17 @@ class PyscalFactory(object):
         return None
 
     @staticmethod
-    def create_wateroilgas_list(relperm_params_df, h=None, fast=False):
+    def create_wateroilgas_list(
+        relperm_params_df: pd.DataFrame, h: Optional[float] = None, fast: bool = False
+    ) -> PyscalList:
         """Create a PyscalList with WaterOilGas objects from
         a dataframe
 
         Args:
-            relperm_params_df (pd.DataFrame): Input data, should have been processed
+            relperm_params_df: Input data, should have been processed
                 through load_relperm_df().
-            h (float): Saturation step-value
-            fast (bool): If fast-mode should be set for constructed object
+            h: Saturation step-value
+            fast: If fast-mode should be set for constructed object
 
         Returns:
             PyscalList, consisting of WaterOilGas objects
@@ -920,15 +966,17 @@ class PyscalFactory(object):
         return wogl
 
     @staticmethod
-    def create_wateroil_list(relperm_params_df, h=None, fast=False):
+    def create_wateroil_list(
+        relperm_params_df: pd.DataFrame, h: Optional[float] = None, fast: bool = False
+    ) -> PyscalList:
         """Create a PyscalList with WaterOil objects from
         a dataframe
 
         Args:
-            relperm_params_df (pd.DataFrame): A valid dataframe with
+            relperm_params_df: A valid dataframe with
                 WaterOil parameters, processed through load_relperm_df()
-            h (float): Saturation steplength
-            fast (bool): If fast-mode should be set for constructed object
+            h: Saturation steplength
+            fast: If fast-mode should be set for constructed object
 
         Returns:
             PyscalList, consisting of WaterOil objects
@@ -946,15 +994,17 @@ class PyscalFactory(object):
         return wol
 
     @staticmethod
-    def create_gasoil_list(relperm_params_df, h=None, fast=False):
+    def create_gasoil_list(
+        relperm_params_df: pd.DataFrame, h: Optional[float] = None, fast: bool = False
+    ) -> PyscalList:
         """Create a PyscalList with GasOil objects from
         a dataframe
 
         Args:
-            relperm_params_df (pd.DataFrame): A valid dataframe with GasOil parameters,
+            relperm_params_df: A valid dataframe with GasOil parameters,
                 processed through load_relperm_df()
-            h (float): Saturation steplength
-            fast (bool): If fast-mode should be set for constructed object
+            h: Saturation steplength
+            fast: If fast-mode should be set for constructed object
 
         Returns:
             PyscalList, consisting of GasOil objects
@@ -972,15 +1022,17 @@ class PyscalFactory(object):
         return gol
 
     @staticmethod
-    def create_gaswater_list(relperm_params_df, h=None, fast=False):
+    def create_gaswater_list(
+        relperm_params_df: pd.DataFrame, h: Optional[float] = None, fast: bool = False
+    ) -> PyscalList:
         """Create a PyscalList with WaterOilGas objects from
         a dataframe, to be used for GasWater
 
         Args:
-            relperm_params_df (pd.DataFrame): A valid dataframe with GasWater
+            relperm_params_df: A valid dataframe with GasWater
                 parameters, processed through load_relperm_df()
-            h (float): Saturation steplength
-            fast (bool): If fast-mode should be set for constructed object
+            h: Saturation steplength
+            fast: If fast-mode should be set for constructed object
 
         Returns:
             PyscalList, consisting of GasWater objects
@@ -998,7 +1050,7 @@ class PyscalFactory(object):
         return gwl
 
 
-def sufficient_water_oil_params(params, failhard=False):
+def sufficient_water_oil_params(params: dict, failhard: bool = False) -> bool:
     """Determine if the supplied parameters are sufficient for
     attempting creating a WaterOil object.
 
@@ -1007,14 +1059,14 @@ def sufficient_water_oil_params(params, failhard=False):
     undefined (Corey or LET, and which pc?)
 
     Args:
-        params (dict): Dictionary of parameters to a WaterOil object.
-        failhard (bool): If True, will raise ValueError when
+        params: Dictionary of parameters to a WaterOil object.
+        failhard: If True, will raise ValueError when
             parameters are insufficient. If defaulted, no
             exception is raised.
 
     Returns:
         True if a WaterOil object should be attempted constructed
-            (but no guarantee for validity of numerical values)
+        (but no guarantee for validity of numerical values)
     """
     # pylint: disable=C1801
     # For case insensitiveness, all keys are converted to lower case:
@@ -1034,7 +1086,7 @@ def sufficient_water_oil_params(params, failhard=False):
     return water_ok and oil_ok
 
 
-def sufficient_gas_oil_params(params, failhard=False):
+def sufficient_gas_oil_params(params: dict, failhard: bool = False) -> bool:
     """Determine if the supplied parameters are sufficient for
     attempting at creating a GasOil object.
 
@@ -1043,14 +1095,14 @@ def sufficient_gas_oil_params(params, failhard=False):
     undefined (Corey or LET, and which pc?)
 
     Args:
-        params (dict): Dictionary of parameters to a GasOil object.
-        failhard (bool): If True, will raise ValueError when
+        params: Dictionary of parameters to a GasOil object.
+        failhard: If True, will raise ValueError when
             parameters are insufficient. If defaulted, no
             exception is raised.
 
     Returns:
         True if a GasOil object should be attempted constructed
-            (but no guarantee for validity of numerical values)
+        (but no guarantee for validity of numerical values)
     """
     # pylint: disable=C1801
     # For case insensitiveness, all keys are converted to lower case:
@@ -1069,7 +1121,7 @@ def sufficient_gas_oil_params(params, failhard=False):
     return oil_ok and gas_ok
 
 
-def sufficient_gas_water_params(params, failhard=False):
+def sufficient_gas_water_params(params: dict, failhard: bool = False) -> bool:
     """Determine if the supplied parameters are sufficient for
     attempting creating a WaterOilGas object to be used for gas water.
 
@@ -1078,8 +1130,8 @@ def sufficient_gas_water_params(params, failhard=False):
     for the factory undefined (Corey or LET, and which pc?)
 
     Args:
-        params (dict): Dictionary of parameters to a GasWater object.
-        failhard (bool): If True, will raise ValueError when
+        params: Dictionary of parameters to a GasWater object.
+        failhard: If True, will raise ValueError when
             parameters are insufficient. If defaulted, no
             exception is raised.
 
@@ -1104,7 +1156,7 @@ def sufficient_gas_water_params(params, failhard=False):
     return water_ok and gas_ok
 
 
-def filter_nan_from_dict(params):
+def filter_nan_from_dict(params: dict) -> dict:
     """Clean out keys with NaN values in a dict.
 
     Key with string values are passed through (empty strings are allowed)
@@ -1125,15 +1177,15 @@ def filter_nan_from_dict(params):
     return cleaned_params
 
 
-def infer_tabular_file_format(filename):
+def infer_tabular_file_format(filename: Union[str, Path]) -> str:
     """Determine the file format of a file containing tabular data,
     distinguishes between csv, xls and xlsx
 
     Args:
-        filename (str): Path to file, possibley pathlib Path object.
+        filename: Path to file
 
     Returns:
-        str: One of "csv", "xlsx" or "xls". Empty string if nothing found out.
+        One of "csv", "xlsx" or "xls". Empty string if nothing found out.
     """
     try:
         pd.read_excel(filename, engine="openpyxl")
@@ -1175,7 +1227,7 @@ def infer_tabular_file_format(filename):
     return ""
 
 
-def check_deprecated(params):
+def check_deprecated(params: Dict[str, Any]) -> None:
     """Check for deprecated parameter names
 
     Args:
