@@ -5,7 +5,16 @@ import math
 from pyscal.constants import MAX_EXPONENT
 
 
-def simple_J(sw, a, b, poro_ref, perm_ref, drho, g=9.81):
+def simple_J(
+    # sw: Union[float, Iterable[float]],
+    sw: float,
+    a: float,
+    b: float,
+    poro_ref: float,
+    perm_ref: float,
+    drho: float,
+    g: float = 9.81,
+) -> float:  # Union[float, Iterable[float]]:
     # pylint: disable=invalid-name,anomalous-backslash-in-string
     r"""Calculate capillary pressure with bar as unit
 
@@ -25,15 +34,15 @@ def simple_J(sw, a, b, poro_ref, perm_ref, drho, g=9.81):
 
 
     Args:
-        sw: float, np.array or similar, water saturation value to be used.
+        sw: float, water saturation value to be used.
             Normalize when needed.
-        a (float): a coefficient
-        b (float): b coefficient
-        poro_ref (float): Reference porosity for scaling to Pc, between 0 and 1
-        perm_ref (float): Reference permeability for scaling to Pc, in milliDarcy
-        drho (float): Density difference between water and oil, in SI units kg/m³.
+        a: a coefficient
+        b: b coefficient
+        poro_ref: Reference porosity for scaling to Pc, between 0 and 1
+        perm_ref: Reference permeability for scaling to Pc, in milliDarcy
+        drho: Density difference between water and oil, in SI units kg/m³.
             Default value is 300
-        g (float): Gravitational acceleration, in SI units m/s²,
+        g: Gravitational acceleration, in SI units m/s²,
             default value is 9.81
 
     Returns:
@@ -50,14 +59,14 @@ def simple_J(sw, a, b, poro_ref, perm_ref, drho, g=9.81):
     return _height_to_pc(height, drho, g)
 
 
-def _height_to_pc(height, drho, g):
+def _height_to_pc(height: float, drho: float, g: float) -> float:
     """From height above free water level, multiplication
     with density difference and gravity gives capillary pressure.
 
     Args:
-        height (float): Height above free water level in meters.
-        drho (float): density difference in g/cc
-        g (float): gravitational acceleration, in m/s²
+        height: Height above free water level in meters.
+        drho: density difference in g/cc
+        g: gravitational acceleration, in m/s²
 
     Returns:
         float: capillary pressure at given height, in bars.
@@ -65,19 +74,19 @@ def _height_to_pc(height, drho, g):
     return height * drho / 1000 * g / 100.0
 
 
-def _sw_to_simpleJ(sw, a, b):
+def _sw_to_simpleJ(sw: float, a: float, b: float) -> float:
     """Convert a water saturation value to the associated J-value,
     using RMS simple-J"""
     return float(a) * sw ** float(b)
 
 
-def _simpleJ_to_sw(J, a, b):
+def _simpleJ_to_sw(J: float, a: float, b: float) -> float:
     """Convert a J-function-value to a water saturation value,
     using RMS simple-J"""
     return math.pow(J / float(a), 1.0 / float(b))
 
 
-def _simpleJ_to_height(J, poro_ref, perm_ref):
+def _simpleJ_to_height(J: float, poro_ref: float, perm_ref: float) -> float:
     """Convert a J-function value to a height-value in meters
 
     This scales the J-value with the inverse of characteristic
@@ -86,14 +95,14 @@ def _simpleJ_to_height(J, poro_ref, perm_ref):
     https://en.wikipedia.org/wiki/Leverett_J-function
 
     Args:
-        J (float): J-function value.
-        poro_ref (float): Porosity between 0 and 1
-        perm_ref (float): Permeability in milliDarcy
+        J: J-function value.
+        poro_ref: Porosity between 0 and 1
+        perm_ref: Permeability in milliDarcy
     """
     return J * math.sqrt(float(poro_ref) / float(perm_ref))
 
 
-def _height_to_simpleJ(H, poro_ref, perm_ref):
+def _height_to_simpleJ(H: float, poro_ref: float, perm_ref: float):
     """Convert a height value (in meters) to a corresponding J-function
 
     This scales the J-value with the characteristic
@@ -102,31 +111,33 @@ def _height_to_simpleJ(H, poro_ref, perm_ref):
     https://en.wikipedia.org/wiki/Leverett_J-function
 
     Args:
-        H (float): Height in meters.
-        poro_ref (float): Porosity between 0 and 1
-        perm_ref (float): Permeability in milliDarcy
+        H: Height in meters.
+        poro_ref: Porosity between 0 and 1
+        perm_ref: Permeability in milliDarcy
     """
     return H * math.sqrt(float(perm_ref) / float(poro_ref))
 
 
-def swl_from_height_simpleJ(swlheight, swirr, a, b, poro_ref, perm_ref):
+def swl_from_height_simpleJ(
+    swlheight: float, swirr: float, a: float, b: float, poro_ref: float, perm_ref: float
+) -> float:
     # pylint: disable=invalid-name
     """Calculate a swl value based on a height parameter.
 
     The height parameter is typically meters above free water level (FWL)
 
     Args:
-        swlheight (float): Height above free water level, in meters,
+        swlheight: Height above free water level, in meters,
             where we want swl to be.
-        swirr (float): Asymptotic irreducible water saturation. This is
+        swirr: Asymptotic irreducible water saturation. This is
             used to normalized the outputted swl.
-        a (float): a coefficient in RMS simplified J function
-        b (float): b coefficient in RMS simplified J function
-        poro_ref (float): Reference porosity for scaling to Pc, between 0 and 1
-        perm_ref (float): Reference permeability for scaling to Pc, in milliDarcy
+        a: a coefficient in RMS simplified J function
+        b: b coefficient in RMS simplified J function
+        poro_ref: Reference porosity for scaling to Pc, between 0 and 1
+        perm_ref: Reference permeability for scaling to Pc, in milliDarcy
 
     Returns:
-        swl (float): A value that can be used for swl.
+        A value that can be used for swl.
     """
     # Calculate the value of J() far above free water level:
     j_value_at_swlheight = _height_to_simpleJ(swlheight, poro_ref, perm_ref)
