@@ -33,6 +33,9 @@ def test_factory_wateroil():
         # pylint: disable=unexpected-keyword-arg
         pyscal_factory.create_water_oil(swirr=0.01)  # noqa
 
+    with pytest.raises(TypeError):
+        pyscal_factory.create_water_oil(params="swirr 0.01")
+
     wateroil = pyscal_factory.create_water_oil(
         dict(
             swirr=0.01,
@@ -49,9 +52,9 @@ def test_factory_wateroil():
     assert wateroil.swirr == 0.01
     assert wateroil.swl == 0.1
     assert wateroil.tag == "Good sand"
-    assert "krw" in wateroil.table
+    assert "KRW" in wateroil.table
     assert "Corey" in wateroil.krwcomment
-    assert wateroil.table["krw"].max() == 0.2  # Because sorw==0 by default
+    assert wateroil.table["KRW"].max() == 0.2  # Because sorw==0 by default
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
     sat_table_str_ok(wateroil.SWFN())
@@ -60,9 +63,9 @@ def test_factory_wateroil():
         dict(nw=3, now=2, sorw=0.1, krwend=0.2, krwmax=0.5)
     )
     assert isinstance(wateroil, WaterOil)
-    assert "krw" in wateroil.table
+    assert "KRW" in wateroil.table
     assert "Corey" in wateroil.krwcomment
-    assert wateroil.table["krw"].max() == 0.5
+    assert wateroil.table["KRW"].max() == 0.5
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
     sat_table_str_ok(wateroil.SWFN())
@@ -70,7 +73,7 @@ def test_factory_wateroil():
     # Ambiguous works, but we don't guarantee that this results
     # in LET or Corey.
     wateroil = pyscal_factory.create_water_oil(dict(nw=3, Lw=2, Ew=2, Tw=2, now=3))
-    assert "krw" in wateroil.table
+    assert "KRW" in wateroil.table
     assert "Corey" in wateroil.krwcomment or "LET" in wateroil.krwcomment
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
@@ -79,8 +82,8 @@ def test_factory_wateroil():
     # Mixing Corey and LET
     wateroil = pyscal_factory.create_water_oil(dict(Lw=2, Ew=2, Tw=2, krwend=1, now=4))
     assert isinstance(wateroil, WaterOil)
-    assert "krw" in wateroil.table
-    assert wateroil.table["krw"].max() == 1.0
+    assert "KRW" in wateroil.table
+    assert wateroil.table["KRW"].max() == 1.0
     assert "LET" in wateroil.krwcomment
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
@@ -90,10 +93,10 @@ def test_factory_wateroil():
         dict(Lw=2, Ew=2, Tw=2, Low=3, Eow=3, Tow=3, krwend=0.5)
     )
     assert isinstance(wateroil, WaterOil)
-    assert "krw" in wateroil.table
-    assert "krow" in wateroil.table
-    assert wateroil.table["krw"].max() == 0.5
-    assert wateroil.table["krow"].max() == 1
+    assert "KRW" in wateroil.table
+    assert "KROW" in wateroil.table
+    assert wateroil.table["KRW"].max() == 0.5
+    assert wateroil.table["KROW"].max() == 1
     assert "LET" in wateroil.krwcomment
     assert "LET" in wateroil.krowcomment
     check_table(wateroil.table)
@@ -104,8 +107,8 @@ def test_factory_wateroil():
     wateroil = pyscal_factory.create_water_oil(
         dict(swl=0.1, nw=1, now=1, a=2, b=-1, poro_ref=0.2, perm_ref=100, drho=200)
     )
-    assert "pc" in wateroil.table
-    assert wateroil.table["pc"].max() > 0.0
+    assert "PC" in wateroil.table
+    assert wateroil.table["PC"].max() > 0.0
     assert "Simplified J" in wateroil.pccomment
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
@@ -115,8 +118,8 @@ def test_factory_wateroil():
     wateroil = pyscal_factory.create_water_oil(
         dict(swl=0.1, nw=1, now=1, a=2, b=-1, poro_ref=0.2, perm_ref=100, drho=200, g=0)
     )
-    assert "pc" in wateroil.table
-    assert wateroil.table["pc"].max() == 0.0
+    assert "PC" in wateroil.table
+    assert wateroil.table["PC"].max() == 0.0
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
     sat_table_str_ok(wateroil.SWFN())
@@ -134,8 +137,8 @@ def test_factory_wateroil():
             drho=200,
         )
     )
-    assert "pc" in wateroil.table
-    assert wateroil.table["pc"].max() > 0.0
+    assert "PC" in wateroil.table
+    assert wateroil.table["PC"].max() > 0.0
     assert "etrophysic" in wateroil.pccomment
     check_table(wateroil.table)
     sat_table_str_ok(wateroil.SWOF())
@@ -145,7 +148,7 @@ def test_factory_wateroil():
     wateroil = pyscal_factory.create_water_oil(
         dict(swl=0.1, nw=1, now=1, a=2, b=-1, perm_ref=100, drho=200, g=0)
     )
-    assert "pc" not in wateroil.table
+    assert "PC" not in wateroil.table
 
 
 def test_fast_mode():
@@ -237,6 +240,22 @@ def test_init_with_swlheight():
             )
         )
 
+    # swlheight must be positive:
+    with pytest.raises(ValueError, match="swlheight must be larger than zero"):
+        pyscal_factory.create_water_oil(
+            dict(
+                swlheight=-200,
+                nw=1,
+                now=1,
+                swirr=0.01,
+                a=1,
+                b=-2,
+                poro_ref=0.2,
+                perm_ref=100,
+                drho=200,
+            )
+        )
+
     # If swcr is large enough, it will pass:
     wateroil = pyscal_factory.create_water_oil(
         dict(
@@ -356,23 +375,31 @@ def test_ambiguity():
     )
     # Corey is picked here.
     assert "Corey" in wateroil.krwcomment
-    assert "krw" in wateroil.table
+    assert "KRW" in wateroil.table
 
 
 def test_factory_gasoil():
     """Test that we can create curves from dictionaries of parameters"""
     pyscal_factory = PyscalFactory()
 
+    # Factory refuses to create incomplete defaulted objects.
+    with pytest.raises(ValueError):
+        pyscal_factory.create_gas_oil()
+
     with pytest.raises(TypeError):
         # (this must be a dictionary)
         # pylint: disable=unexpected-keyword-arg
         pyscal_factory.create_gas_oil(swirr=0.01)  # noqa
 
+    with pytest.raises(TypeError):
+        pyscal_factory.create_gas_oil(params="swirr 0.01")
+
     gasoil = pyscal_factory.create_gas_oil(
         dict(swirr=0.01, swl=0.1, sgcr=0.05, tag="Good sand", ng=1, nog=2)
     )
     assert isinstance(gasoil, GasOil)
     assert gasoil.sgcr == 0.05
+    assert gasoil.sgro == 0.0
     assert gasoil.swl == 0.1
     assert gasoil.swirr == 0.01
     assert gasoil.tag == "Good sand"
@@ -407,14 +434,105 @@ def test_factory_gasoil():
     assert "LET krog" in sgof
 
 
+def test_factory_wog_gascondensate():
+    """Test modelling of gas condensate, which in pyscal terms
+    is the same as wateroilgas, except that we allow for aliasing
+    in sgrw=sorw for the underlying WaterOil object, and also there
+    are additional parameters sgro and kromax for GasOil."""
+    wcg = PyscalFactory.create_water_oil_gas(
+        dict(
+            nw=2,
+            now=3,
+            ng=1,
+            nog=2,
+            sgrw=0.1,
+            swl=0.1,
+            sgcr=0.1,
+            sgro=0.1,
+            kroend=0.5,
+            kromax=0.9,
+        )
+    )
+    assert wcg.gasoil.sgro == 0.1
+    assert wcg.wateroil.sorw == 0.1
+
+    swof = wcg.SWOF()
+    sgof = wcg.SGOF()
+
+    # sgrw has been aliased to sorw, but the WaterOil object does not know that:
+    assert "sgrw" not in swof
+    assert "sorw=0.1" in swof
+    assert "sgro=0.1" in sgof
+    assert "kroend=0.5" in sgof
+    assert "kromax=0.9" in sgof
+
+    sat_table_str_ok(swof)
+    sat_table_str_ok(sgof)
+
+    # Different sorw and sgrw is a hard error:
+    with pytest.raises(ValueError, match="must equal"):
+        PyscalFactory.create_water_oil_gas(
+            dict(nw=2, now=3, ng=1, nog=2, sorw=0.2, sgrw=0.1, swl=0.1)
+        )
+
+    # But it will pass if they both are supplied but are equal:
+    wcg_2 = PyscalFactory.create_water_oil_gas(
+        dict(nw=2, now=3, ng=1, nog=2, sorw=0.2, sgrw=0.2, swl=0.1)
+    )
+    assert "sorw=0.2" in wcg_2.SWOF()
+
+    # kroend higher than kromax is an error:
+    with pytest.raises(AssertionError):
+        PyscalFactory.create_water_oil_gas(
+            dict(
+                nw=2,
+                now=3,
+                ng=1,
+                nog=2,
+                sgcr=0.1,
+                sgro=0.1,
+                kromax=0.5,
+                kroend=0.8,
+                swl=0.1,
+            )
+        )
+
+
+def test_factory_go_gascondensate():
+    """In gas condensate problems, the sgro and kromax parameters are relevant"""
+    pyscal_factory = PyscalFactory()
+    gasoil = pyscal_factory.create_gas_oil(
+        dict(sgro=0.1, sgcr=0.1, tag="Good sand", ng=1, nog=2, kroend=0.5, kromax=0.9)
+    )
+    assert isinstance(gasoil, GasOil)
+    assert gasoil.sgro == 0.1
+    assert gasoil.tag == "Good sand"
+    sgof = gasoil.SGOF()
+    sat_table_str_ok(sgof)
+    check_table(gasoil.table)
+    assert "Corey krog" in sgof
+    assert "kroend=0.5" in sgof
+    assert "kromax=0.9" in sgof
+    assert "sgro=0.1" in sgof
+
+
 def test_factory_gaswater():
     """Test that we can create gas-water curves from dictionaries of parameters"""
     pyscal_factory = PyscalFactory()
 
+    # Factory refuses to create incomplete defaulted objects.
+    with pytest.raises(ValueError):
+        pyscal_factory.create_gas_water()
+
     with pytest.raises(TypeError):
         # pylint: disable=unexpected-keyword-arg
         pyscal_factory.create_gas_water(swirr=0.01)  # noqa
 
+    with pytest.raises(TypeError):
+        # (it must be a dictionary)
+        # pylint: disable=unexpected-keyword-arg
+        pyscal_factory.create_gas_water(params="swirr 0.01")
+
     gaswater = pyscal_factory.create_gas_water(
         dict(swirr=0.01, swl=0.03, sgrw=0.1, sgcr=0.15, tag="gassy sand", ng=2, nw=2)
     )
@@ -457,6 +575,18 @@ def test_factory_wateroilgas():
     """Test creating discrete cases of WaterOilGas from factory"""
     pyscal_factory = PyscalFactory()
 
+    # Factory refuses to create incomplete defaulted objects.
+    with pytest.raises(ValueError):
+        pyscal_factory.create_water_oil_gas()
+
+    with pytest.raises(TypeError):
+        # (this must be a dictionary)
+        # pylint: disable=unexpected-keyword-arg
+        pyscal_factory.create_water_oil_gas(swirr=0.01)  # noqa
+
+    with pytest.raises(TypeError):
+        pyscal_factory.create_water_oil_gas(params="swirr 0.01")
+
     wog = pyscal_factory.create_water_oil_gas(dict(nw=2, now=3, ng=1, nog=2.5))
     swof = wog.SWOF()
     sgof = wog.SGOF()
@@ -513,7 +643,7 @@ def test_factory_wateroilgas_wo():
     )
     swof = wog.SWOF()
     assert "Corey krw" in swof
-    assert "krw" in wog.wateroil.table
+    assert "KRW" in wog.wateroil.table
     sat_table_str_ok(swof)
     check_table(wog.wateroil.table)
     assert wog.gasoil is None
@@ -521,7 +651,7 @@ def test_factory_wateroilgas_wo():
     wog.SGOF()
 
 
-def test_load_relperm_df(tmpdir):
+def test_load_relperm_df(tmpdir, caplog):
     """Test loading of dataframes with validation from excel or from csv"""
     testdir = Path(__file__).absolute().parent
 
@@ -531,6 +661,9 @@ def test_load_relperm_df(tmpdir):
     with pytest.raises(IOError):
         PyscalFactory.load_relperm_df("not-existing-file")
 
+    with pytest.raises(ValueError, match="Non-existing sheet-name"):
+        PyscalFactory.load_relperm_df(scalfile_xls, sheet_name="foo")
+
     assert "SATNUM" in scaldata
     assert "CASE" in scaldata
     assert not scaldata.empty
@@ -545,6 +678,12 @@ def test_load_relperm_df(tmpdir):
     assert "SATNUM" in scaldata_fromdf
     assert len(scaldata_fromdf) == len(scaldata_fromcsv) == len(scaldata)
 
+    scaldata_fromcsv = PyscalFactory.load_relperm_df("scal-input.csv", sheet_name="foo")
+    assert "Sheet name only relevant for XLSX files, ignoring foo" in caplog.text
+
+    with pytest.raises(ValueError, match="Unsupported argument"):
+        PyscalFactory.load_relperm_df(dict(foo=1))
+
     # Perturb the dataframe, this should trigger errors
     with pytest.raises(ValueError):
         PyscalFactory.load_relperm_df(scaldata.drop("SATNUM", axis="columns"))
@@ -584,11 +723,43 @@ def test_load_relperm_df(tmpdir):
     assert "foobar" in swof_str  # Random string injected in xlsx.
 
     # Make a dummy text file
-    with open("dummy.txt", "w") as fhandle:
-        fhandle.write("foo\nbar, com")
+    Path("dummy.txt").write_text("foo\nbar, com")
     with pytest.raises(ValueError):
         PyscalFactory.load_relperm_df("dummy.txt")
 
+    # Make an empty csv file
+    Path("empty.csv").write_text("")
+    # A ValueError would also make sense here
+    assert PyscalFactory.load_relperm_df("empty.csv").empty
+
+    # Merge tags and comments if both are supplied
+    Path("tagandcomment.csv").write_text(
+        "SATNUM,nw,now,tag,comment\n1,1,1,a-tag,a-comment"
+    )
+    tagandcomment_df = PyscalFactory.load_relperm_df("tagandcomment.csv")
+    assert (
+        tagandcomment_df["TAG"].values[0] == "SATNUM 1 tag: a-tag; comment: a-comment"
+    )
+
+    # Missing SATNUMs:
+    Path("wrongsatnum.csv").write_text("SATNUM,nw,now\n1,1,1\n3,1,1")
+    with pytest.raises(ValueError, match="Missing SATNUMs?"):
+        PyscalFactory.load_relperm_df("wrongsatnum.csv")
+
+    # Missing SATNUMs, like merged cells:
+    Path("mergedcells.csv").write_text(
+        "CASE,SATNUM,nw,now\nlow,,1,1\nlow,1,2,2\nlow,,3,32"
+    )
+    with pytest.raises(ValueError, match="Found not-a-number"):
+        PyscalFactory.load_relperm_df("mergedcells.csv")
+
+    # Missing SATNUMs, like merged cells:
+    Path("mergedcellscase.csv").write_text(
+        "CASE,SATNUM,nw,now\n,1,1,1\nlow,1,2,2\n,1,3,32"
+    )
+    with pytest.raises(ValueError, match="Found not-a-number"):
+        PyscalFactory.load_relperm_df("mergedcellscase.csv")
+
 
 def test_many_nans():
     """Excel or oocalc sometimes saves a xlsx file that gives all NaN rows and
@@ -674,6 +845,10 @@ def test_scalrecommendation():
         "high": {"nw": 4, "now": 2, "ng": 1, "nog": 3},
     }
     scal = pyscal_factory.create_scal_recommendation(scal_input)
+
+    with pytest.raises(ValueError, match="Input must be a dict"):
+        pyscal_factory.create_scal_recommendation("low")
+
     # (not supported yet to make WaterOil only..)
     interp = scal.interpolate(-0.5)
     sat_table_str_ok(interp.SWOF())
@@ -683,10 +858,12 @@ def test_scalrecommendation():
     check_table(interp.wateroil.table)
     check_table(interp.gasoil.table)
 
-    incomplete1 = scal_input.copy()
-    del incomplete1["BASE"]
-    with pytest.raises(ValueError):
-        pyscal_factory.create_scal_recommendation(incomplete1)
+    # Check that we error if any of the parameters above is missing:
+    for case in ["low", "BASE", "high"]:
+        copy1 = scal_input.copy()
+        del copy1[case]
+        with pytest.raises(ValueError):
+            pyscal_factory.create_scal_recommendation(copy1)
 
     go_only = scal_input.copy()
     del go_only["low"]["now"]
@@ -700,6 +877,16 @@ def test_scalrecommendation():
     assert "SGFN" in gasoil.interpolate(-0.4).SGFN()
     assert "SWOF" not in gasoil.interpolate(-0.2).SWOF()
 
+    basehigh = scal_input.copy()
+    del basehigh["low"]
+    with pytest.raises(ValueError, match='"low" case not supplied'):
+        pyscal_factory.create_scal_recommendation(basehigh)
+
+    baselow = scal_input.copy()
+    del baselow["high"]
+    with pytest.raises(ValueError, match='"high" case not supplied'):
+        pyscal_factory.create_scal_recommendation(baselow)
+
 
 def test_scalrecommendation_gaswater():
     """Testing making SCAL rec from dict of dict for gaswater input"""
@@ -724,12 +911,9 @@ def test_xls_scalrecommendation():
 
     xlsxfile = testdir / "data/scal-pc-input-example.xlsx"
     scalinput = pd.read_excel(xlsxfile, engine="openpyxl").set_index(["SATNUM", "CASE"])
-    print(scalinput)
     for satnum in scalinput.index.levels[0].values:
         dictofdict = scalinput.loc[satnum, :].to_dict(orient="index")
-        print(dictofdict)
         scalrec = PyscalFactory.create_scal_recommendation(dictofdict)
-        print(scalrec.interpolate(-0.5).SWOF())
         scalrec.interpolate(+0.5)
 
 
@@ -752,27 +936,12 @@ def test_check_deprecated_krowgend(caplog):
     wateroil = PyscalFactory.create_water_oil(dict(swl=0.1, nw=2, now=2, krowend=0.4))
     assert "krowend" in caplog.text
     assert "deprecated" in caplog.text
-    assert wateroil.table["krow"].max() == 0.4
+    assert wateroil.table["KROW"].max() == 0.4
 
     gasoil = PyscalFactory.create_gas_oil(dict(swl=0.1, ng=2, nog=2, krogend=0.4))
     assert "krogend" in caplog.text
     assert "deprecated" in caplog.text
-    assert gasoil.table["krog"].max() == 0.4
-
-
-def test_check_deprecated_kromax(caplog):
-    """Up until pyscal 0.5.x, kromax was a parameter to the oil
-    curve parametrization for WaterOil and GasOil, and kro was
-    linear between swl and swcr. From pyscal 0.6, that linear
-    segment is gone, and kromax is not needed as a parameter,
-    only krowend/krogend is used.
-
-    If kromax is encountered in input data, we should warn
-    it is ignored.
-    """
-    PyscalFactory.create_water_oil(dict(swl=0.1, nw=2, now=2, kroend=0.4, kromax=0.5))
-    assert "kromax" in caplog.text
-    assert "deprecated" in caplog.text
+    assert gasoil.table["KROG"].max() == 0.4
 
 
 def parse_gensatfuncline(conf_line):
@@ -1074,3 +1243,17 @@ def test_infer_tabular_file_format(tmpdir, caplog):
     Path("wrong.csv").write_bytes(os.urandom(100))
     assert factory.infer_tabular_file_format("wrong.csv") == ""
     assert "Impossible to infer file format for wrong.csv" in caplog.text
+
+
+@pytest.mark.parametrize(
+    "orig_dict, keylist, expected_dict",
+    [
+        ({}, [], {}),
+        ({"foo": 1}, [], {}),
+        ({"foo": 1}, ["fo"], {}),
+        ({"foo": 1}, ["foo"], {"foo": 1}),
+        ({}, ["foo"], {}),
+    ],
+)
+def test_slicedict(orig_dict, keylist, expected_dict):
+    assert factory.slicedict(orig_dict, keylist) == expected_dict
