@@ -5,6 +5,7 @@ import pandas as pd
 
 from matplotlib import pyplot as plt
 
+import pytest
 from hypothesis import given, settings
 import hypothesis.strategies as st
 
@@ -74,8 +75,8 @@ def test_normalize_pc(swirr, dswl):
     """Test that we can normalize a pc curve"""
     wateroil = WaterOil(swirr=swirr, swl=swirr + dswl)
     wateroil.add_simple_J()
-    pc_max = wateroil.table["pc"].max()
-    pc_min = wateroil.table["pc"].min()
+    pc_max = wateroil.table["PC"].max()
+    pc_min = wateroil.table["PC"].min()
 
     pc_fn = normalize_pc(wateroil)
     assert np.isclose(pc_fn(0), pc_max)
@@ -149,7 +150,7 @@ def test_normalize_nonlinpart_wo():
     assert np.isclose(kron(1), 0.8)
 
     # So fix endpoints!
-    wateroil.swl = wateroil.table["sw"].min()
+    wateroil.swl = wateroil.table["SW"].min()
     wateroil.swcr = wateroil.estimate_swcr()
     wateroil.sorw = wateroil.estimate_sorw()
     # Try again
@@ -273,16 +274,16 @@ def test_interpolate_wo(
 
     # Distances between low and interpolants:
     dists = [
-        (wo_low.table - interp.table)[["krw", "krow"]].sum().sum() for interp in ips
+        (wo_low.table - interp.table)[["KRW", "KROW"]].sum().sum() for interp in ips
     ]
     assert np.isclose(dists[0], 0)
 
     # Distance between high and the last interpolant
-    assert (wo_high.table - ips[-1].table)[["krw", "krow"]].sum().sum() < 0.01
+    assert (wo_high.table - ips[-1].table)[["KRW", "KROW"]].sum().sum() < 0.01
 
     # Distances between low and interpolants:
     dists = [
-        (wo_low.table - interp.table)[["krw", "krow"]].sum().sum() for interp in ips
+        (wo_low.table - interp.table)[["KRW", "KROW"]].sum().sum() for interp in ips
     ]
     print(
         "Interpolation, mean: {}, min: {}, max: {}, std: {} ip-par-dist: {}".format(
@@ -349,14 +350,14 @@ def test_interpolate_wo_pc(swl, dswcr, dswlhigh, sorw, a_l, a_h, b_l, b_h):
         assert 0 < wo_ip.crosspoint() < 1
 
     # Distances between low and interpolants:
-    dists = [(wo_low.table - interp.table)[["pc"]].sum().sum() for interp in ips]
+    dists = [(wo_low.table - interp.table)[["PC"]].sum().sum() for interp in ips]
     assert np.isclose(dists[0], 0)
 
     # Distance between high and the last interpolant
-    assert (wo_high.table - ips[-1].table)[["pc"]].sum().sum() < 0.01
+    assert (wo_high.table - ips[-1].table)[["PC"]].sum().sum() < 0.01
 
     # Distances between low and interpolants:
-    dists = [(wo_low.table - interp.table)[["pc"]].sum().sum() for interp in ips]
+    dists = [(wo_low.table - interp.table)[["PC"]].sum().sum() for interp in ips]
     print(
         "Interpolation, mean: {}, min: {}, max: {}, std: {} ip-par-dist: {}".format(
             np.mean(dists), min(dists), max(dists), np.std(np.diff(dists[1:])), ip_dist
@@ -391,14 +392,17 @@ def test_interpolate_wo_pc(swl, dswcr, dswlhigh, sorw, a_l, a_h, b_l, b_h):
     st.floats(min_value=0, max_value=0.1),  # dsgcr
     st.floats(min_value=0, max_value=0.3),  # sorg
     st.floats(min_value=0, max_value=0.1),  # dsorg
+    st.booleans(),  # sgrononzero
     st.floats(min_value=0.1, max_value=5),  # ng1
     st.floats(min_value=0.01, max_value=1),  # krgend1
     st.floats(min_value=0.1, max_value=5),  # nog1
     st.floats(min_value=0.01, max_value=1),  # kroend1
+    st.floats(min_value=0.01, max_value=1),  # kromax1
     st.floats(min_value=0.1, max_value=5),  # ng2
     st.floats(min_value=0.01, max_value=1),  # krgend2
     st.floats(min_value=0.1, max_value=5),  # nog2
     st.floats(min_value=0.01, max_value=1),  # kroend2
+    st.floats(min_value=0.01, max_value=1),  # kromax2
 )
 def test_normalize_nonlinpart_go_hypo(
     swl,
@@ -407,26 +411,44 @@ def test_normalize_nonlinpart_go_hypo(
     dsgcr,
     sorg,
     dsorg,
+    sgrononzero,
     ng1,
     krgend1,
     nog1,
     kroend1,
+    kromax1,
     ng2,
     krgend2,
     nog2,
     kroend2,
+    kromax2,
 ):
     # pylint: disable=too-many-arguments,too-many-locals
     """Test the normalization code in utils.
 
     In particular the fill_value argument to scipy has been tuned to
     fulfill this code"""
-    go_low = GasOil(swl=swl, sgcr=sgcr, sorg=sorg)
-    go_high = GasOil(swl=swl + dswlhigh, sgcr=sgcr + dsgcr, sorg=max(sorg - dsorg, 0))
+    kroend1 = min(kroend1, kromax1)
+    kroend2 = min(kroend2, kromax2)
+
+    if sgrononzero:
+        sgro_low = sgcr
+        sgro_high = sgcr + dsgcr
+    else:
+        sgro_low = 0.0
+        sgro_high = 0.0
+
+    go_low = GasOil(swl=swl, sgcr=sgcr, sorg=sorg, sgro=sgro_low)
+    go_high = GasOil(
+        swl=swl + dswlhigh,
+        sgcr=sgcr + dsgcr,
+        sgro=sgro_high,
+        sorg=max(sorg - dsorg, 0),
+    )
     go_low.add_corey_gas(ng=ng1, krgend=krgend1)
     go_high.add_corey_gas(ng=ng2, krgend=krgend2)
-    go_low.add_corey_oil(nog=nog1, kroend=kroend1)
-    go_high.add_corey_oil(nog=nog2, kroend=kroend2)
+    go_low.add_corey_oil(nog=nog1, kroend=kroend1, kromax=kromax1)
+    go_high.add_corey_oil(nog=nog2, kroend=kroend2, kromax=kromax2)
 
     krgn1, kron1 = normalize_nonlinpart_go(go_low)
     assert np.isclose(krgn1(0), 0)
@@ -443,9 +465,9 @@ def test_normalize_nonlinpart_go_hypo(
 
 def test_normalize_nonlinpart_go():
     """Manual tests for normalize_nonlinpart_go"""
-    gasoil = GasOil(swl=0.1, sgcr=0.12, sorg=0.05, h=0.05)
+    gasoil = GasOil(swl=0.1, sgcr=0.12, sgro=0.12, sorg=0.05, h=0.05)
     gasoil.add_corey_gas(ng=2.1, krgend=0.9)
-    gasoil.add_corey_oil(nog=3, kroend=0.8)
+    gasoil.add_corey_oil(nog=3, kroend=0.75, kromax=0.8)
     krgn, kron = normalize_nonlinpart_go(gasoil)
 
     assert np.isclose(krgn(0), 0)
@@ -453,7 +475,7 @@ def test_normalize_nonlinpart_go():
 
     # kron is normalized on son
     assert np.isclose(kron(0), 0)
-    assert np.isclose(kron(1), 0.8)
+    assert np.isclose(kron(1), 0.75)
 
     # Test with tricky endpoints
     h = 0.01
@@ -492,7 +514,7 @@ def test_normalize_nonlinpart_go():
     assert not np.isclose(krgn(1), 0.6)
 
     # So fix endpoints!
-    gasoil.swl = 1 - gasoil.table["sg"].max()
+    gasoil.swl = 1 - gasoil.table["SG"].max()
     gasoil.sgcr = gasoil.estimate_sgcr()
     gasoil.sorg = gasoil.estimate_sorg()
     # Try again
@@ -517,10 +539,31 @@ def test_ip_wo_kroend():
     wo_ip = interpolate_wo(wo_low, wo_high, 0.5)
 
     # kroend at mean swl:
-    assert float_df_checker(wo_ip.table, "sw", 0.01, "krow", (0.6 + 0.7) / 2.0)
+    assert float_df_checker(wo_ip.table, "SW", 0.01, "KROW", (0.6 + 0.7) / 2.0)
 
-    assert float_df_checker(wo_ip.table, "sw", 1, "krw", 0.71)
-    assert float_df_checker(wo_ip.table, "sw", 1 - 0.15, "krw", 0.5)
+    assert float_df_checker(wo_ip.table, "SW", 1, "KRW", 0.71)
+    assert float_df_checker(wo_ip.table, "SW", 1 - 0.15, "KRW", 0.5)
+
+
+def test_ip_go_kroendmax():
+    """Test behaviour of kroend/kromax under interpolation, gas condensate modelling"""
+    go_low = GasOil(swl=0, sgro=0.1, sgcr=0.1)
+    go_high = GasOil(swl=0, sgro=0)
+    go_low.add_corey_gas()
+    go_low.add_corey_oil(nog=2, kroend=0.5, kromax=1)
+    go_high.add_corey_gas()
+    go_high.add_corey_oil(nog=2, kroend=1)
+
+    # Interpolate to midpoint between the curves above
+    go_ip = interpolate_go(go_low, go_high, 0.5)
+
+    # kro(sg=0) is 1 for all interpolants:
+    assert float_df_checker(go_ip.table, "SG", 0.0, "KROG", 1.0)
+
+    # kro(sg=mean(sgro)) = mean kroeend
+    assert float_df_checker(go_ip.table, "SG", (0 + 0.1) / 2.0, "KROG", 0.75)
+
+    assert np.isclose(go_ip.estimate_sgro(), (0 + 0.1) / 2.0)
 
 
 def test_ip_go_kroend():
@@ -537,15 +580,15 @@ def test_ip_go_kroend():
     go_ip = interpolate_go(go_low, go_high, 0.5)
 
     # kroend at sg zero:
-    assert float_df_checker(go_ip.table, "sg", 0.0, "krog", (0.6 + 0.7) / 2.0)
+    assert float_df_checker(go_ip.table, "SG", 0.0, "KROG", (0.6 + 0.7) / 2.0)
 
     assert np.isclose(go_ip.swl, 0.01)
     assert np.isclose(go_ip.sorg, 0.15)
 
     # krgmax at 1 - swl:
-    assert float_df_checker(go_ip.table, "sg", 1 - go_ip.swl, "krg", 0.71)
+    assert float_df_checker(go_ip.table, "SG", 1 - go_ip.swl, "KRG", 0.71)
     # krgend at 1 - swl - sorg
-    assert float_df_checker(go_ip.table, "sg", 1 - go_ip.swl - go_ip.sorg, "krg", 0.5)
+    assert float_df_checker(go_ip.table, "SG", 1 - go_ip.swl - go_ip.sorg, "KRG", 0.5)
 
     # If krgendanchor is None, then krgmax should be irrelevant
     go_low = GasOil(swl=0, sgcr=0.1, sorg=0.2, krgendanchor="")
@@ -562,7 +605,7 @@ def test_ip_go_kroend():
     # Interpolate to midpoint between the curves above
     go_ip = interpolate_go(go_low, go_high, 0.5, h=0.1)
 
-    assert float_df_checker(go_ip.table, "sg", 0.0, "krog", (0.6 + 0.7) / 2.0)
+    assert float_df_checker(go_ip.table, "SG", 0.0, "KROG", (0.6 + 0.7) / 2.0)
 
     # Activate these line to see a bug, interpolation_go
     # does not honor krgendanchorA:
@@ -573,7 +616,7 @@ def test_ip_go_kroend():
     # plt.show()
 
     # krgmax is irrelevant, krgend is used here:
-    assert float_df_checker(go_ip.table, "sg", 1 - 0.01, "krg", 0.5)
+    assert float_df_checker(go_ip.table, "SG", 1 - 0.01, "KRG", 0.5)
 
     # Also check that estimated new sgcr is between the inputs:
     assert 0.05 <= go_ip.estimate_sgcr() <= 0.1
@@ -590,17 +633,17 @@ def test_ip_go_kroend():
     # Interpolate to midpoint between the curves above
     go_ip = interpolate_go(go_low, go_high, 0.5)
 
-    assert float_df_checker(go_ip.table, "sg", 0.0, "krog", (0.6 + 0.7) / 2.0)
+    assert float_df_checker(go_ip.table, "SG", 0.0, "KROG", (0.6 + 0.7) / 2.0)
 
     # max(krg) is here avg of krgmax and krgend from the differnt tables:
-    assert float_df_checker(go_ip.table, "sg", 1 - 0.01, "krg", 0.6)
+    assert float_df_checker(go_ip.table, "SG", 1 - 0.01, "KRG", 0.6)
 
     # krgend at 1 - swl - sorg, non-trivial expression, so a numerical
     # value is used here in the test:
-    assert float_df_checker(go_ip.table, "sg", 1 - 0.01 - 0.15, "krg", 0.4491271)
+    assert float_df_checker(go_ip.table, "SG", 1 - 0.01 - 0.15, "KRG", 0.4491271)
 
     # krog-zero at 1 - swl - sorg:
-    assert float_df_checker(go_ip.table, "sg", 1 - 0.01 - 0.15, "krog", 0)
+    assert float_df_checker(go_ip.table, "SG", 1 - 0.01 - 0.15, "KROG", 0)
 
 
 @settings(max_examples=50, deadline=5000)
@@ -678,7 +721,7 @@ def test_interpolate_go(
 
     # Distances between low and interpolants:
     dists = [
-        (go_low.table - interp.table)[["krg", "krog"]].sum().sum() for interp in ips
+        (go_low.table - interp.table)[["KRG", "KROG"]].sum().sum() for interp in ips
     ]
     print(
         "Interpolation, mean: {}, min: {}, max: {}, std: {} ip-par-dist: {}".format(
@@ -714,7 +757,7 @@ def test_interpolations_go_fromtable():
     was underestimated in interpolations following add_fromtable().
     """
     base = pd.DataFrame(
-        columns=["Sg", "krg", "krog"],
+        columns=["SG", "KRG", "KROG"],
         data=[
             [0.0, 0.0, 1.0],
             [0.1, 0.0, 1.0],
@@ -726,7 +769,7 @@ def test_interpolations_go_fromtable():
         ],
     )
     opt = pd.DataFrame(
-        columns=["Sg", "krg", "krog"],
+        columns=["SG", "KRG", "KROG"],
         data=[
             [0.0, 0.0, 1.0],
             [0.1, 0.0, 1.0],
@@ -757,7 +800,7 @@ def test_interpolations_wo_fromtable():
     Pyscal 0.6.1 and earlier fails this test sorw.
     """
     base = pd.DataFrame(
-        columns=["Sw", "krw", "krow"],
+        columns=["SW", "KRW", "KROW"],
         data=[
             [0.0, 0.0, 1.0],
             [0.1, 0.0, 1.0],
@@ -769,7 +812,7 @@ def test_interpolations_wo_fromtable():
         ],
     )
     opt = pd.DataFrame(
-        columns=["Sw", "krw", "krow"],
+        columns=["SW", "KRW", "KROW"],
         data=[
             [0.0, 0.0, 1.0],
             [0.1, 0.0, 1.0],
@@ -792,3 +835,50 @@ def test_interpolations_wo_fromtable():
     wo_ip = interpolate_wo(wo_base, wo_opt, 0.5, h=0.01)
     assert np.isclose(wo_ip.estimate_swcr(), 0.25)
     assert np.isclose(wo_ip.estimate_sorw(), 0.15)
+
+
+@given(
+    st.floats(min_value=0, max_value=0.1),  # sgro_low
+    st.floats(min_value=0, max_value=0.1),  # sgro_high
+)
+def test_gascond_interpolation(sgro_low, sgro_high):
+    """sgro is required to be either 0 or sgcr, and interpolations
+    will crash when this is not the case. This test validates that
+    we can let sgro and sgcr go to zero, and that we always are able
+    to interpolate without crashes."""
+    go_low = GasOil(sgro=sgro_low, sgcr=sgro_low)
+    go_high = GasOil(sgro=sgro_high, sgcr=sgro_high)
+    go_low.add_corey_gas()
+    go_low.add_corey_oil()
+    go_high.add_corey_gas()
+    go_high.add_corey_oil()
+
+    go_ip = interpolate_go(go_low, go_high, parameter=0.5)
+    check_table(go_ip.table)
+
+
+def test_gasoil_gascond_fails():
+    """Interpolation between an object for gas condendensate (sgro > 0)
+    and an object with sgro = 0 (regular gas-oil) should fail"""
+    gascond = GasOil(sgro=0.1, sgcr=0.1)
+    gasoil = GasOil(sgro=0.0, sgcr=0.1)
+
+    gasoil.add_corey_gas()
+    gasoil.add_corey_oil()
+    gascond.add_corey_gas()
+    gascond.add_corey_oil()
+
+    # interpolation parameter at zero is no interpolation, just lookup,
+    # so it works fine:
+    check_table(interpolate_go(gasoil, gascond, parameter=0.0).table)
+    check_table(interpolate_go(gasoil, gascond, parameter=epsilon).table)
+    check_table(interpolate_go(gasoil, gascond, parameter=10 * epsilon).table)
+
+    with pytest.raises(ValueError, match="Interpolated sgro"):
+        interpolate_go(gasoil, gascond, parameter=100 * epsilon)
+
+    with pytest.raises(ValueError, match="Interpolated sgro"):
+        interpolate_go(gasoil, gascond, parameter=0.5)
+
+    check_table(interpolate_go(gasoil, gascond, parameter=1.0 - epsilon).table)
+    check_table(interpolate_go(gasoil, gascond, parameter=1.0).table)
